# -*- coding: utf-8 -*-

# Copyright (c) Pilot Systems, 2013

# This file is part of Faceted SeSQL.

# SeSQL is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# SeSQL is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with Faceted SeSQL.  If not, see <http://www.gnu.org/licenses/>.

from search import FacetedSearch
from django.core.paginator import Paginator, EmptyPage, PageNotAnInteger
from django.conf import settings

def faceted_from_request(config, request, external_filter = None):
    """
    Build a faceted search from request
    """
    search = FacetedSearch(config)
    for facet in search.facets():
        fltr = facet.facet.get_filter_from_request(request)
        if fltr:
            search.set_criteria(facet.facet.name, fltr)

    if external_filter:
        search.apply_external_filter(external_filter)
        
    order = request.GET.get('order_by', None)
    if order:
        search.set_order(order)

    paginator = Paginator(tuple(search), settings.ITEMS_PER_PAGE)
    page = request.GET.get('page')

    try:
        items = paginator.page(page)
    except PageNotAnInteger:
        items = paginator.page(1)
    except EmptyPage:
        items = paginator.page(paginator.num_pages)


    search.results = items

    return search

def pagination (page):

    begin_pages=2
    end_pages=2
    before_current_pages=3
    after_current_pages=3

    before = max(page.number - before_current_pages - 1, 0)
    after = page.number + after_current_pages

    begin = page.paginator.page_range[:begin_pages]
    middle = page.paginator.page_range[before:after]
    end = page.paginator.page_range[-end_pages:]
    last_page_number = end[-1]

    def collides(firstlist, secondlist):
        return any(item in secondlist for item in firstlist)

    if collides(middle, end):
        end = range(max(page.number-before_current_pages, 1), last_page_number+1)

        middle = []
        
    if collides(begin, middle):
        begin = range(1, min(page.number + after_current_pages, last_page_number)+1)

        middle = []
        
    if collides(begin, end):
        begin = range(1, last_page_number+1)
        end = []

    return begin, middle, end
